import ktc
import os
import cv2
import math
import torch
import numpy as np


## Test Various converters.

## Clone https://github.com/kneron/ConvertorExamples first:
## git clone https://github.com/kneron/ConvertorExamples.git
## cd ConvertorExamples && git lfs pull

## Adjust loggin setting to avoid too many messages before start.
# import logging
# logging.basicConfig(level=logging.INFO)

## Keras to ONNX
# result_m = ktc.onnx_optimizer.keras2onnx_flow('/data1/ConvertorExamples/keras_example/onet-0.417197.hdf5')

## Pytorch to ONNX
# import torch
# import torch.onnx
# # Load the pth saved model
# pth_model = torch.load("/data1/ConvertorExamples/pytorch_example/resnet34.pth", map_location='cpu')
# # Export the model
# dummy_input = torch.randn(1, 3, 224, 224)
# torch.onnx.export(pth_model, dummy_input, '/data1/resnet34.onnx', opset_version=11)
# # Load the exported onnx model as an onnx object
# exported_m = onnx.load('/data1/resnet34.onnx')
# # Optimize the exported onnx object
# result_m = ktc.onnx_optimizer.torch_exported_onnx_flow(exported_m)

## Caffe to ONNX
# result_m = ktc.onnx_optimizer.caffe2onnx_flow('/data1/ConvertorExamples/caffe_example/mobilenetv2.prototxt', '/data1/ConvertorExamples/caffe_example/mobilenetv2.caffemodel')

## TF Lite to ONNX
# result_m = ktc.onnx_optimizer.tflite2onnx_flow('/data1/ConvertorExamples/tflite_example/model_unquant.tflite')

## ONNX Optimization
# optimized_m = ktc.onnx_optimizer.onnx2onnx_flow(result_m, eliminate_tail=True)

## Section 3
# km = ktc.ModelConfig(1001, "0001", "720", onnx_path="/data1/output1.onnx")
# eval_result = km.evaluate()

km = ktc.ModelConfig(1001, "0001", "720", onnx_path="/data1/output2.onnx")
eval_result = km.evaluate()

stride = 8
height_size = 256

def normalize(img, img_mean, img_scale):
    img = np.array(img, dtype=np.float32)
    img = (img - img_mean) * img_scale
    return img

# def pad_width(img, stride, pad_value, min_dims):
#     h, w, _ = img.shape
#     h = min(min_dims[0], h)
#     min_dims[0] = math.ceil(min_dims[0] / float(stride)) * stride
#     min_dims[1] = max(min_dims[1], w)
#     min_dims[1] = math.ceil(min_dims[1] / float(stride)) * stride
#     pad = []
#     pad.append(int(math.floor((min_dims[0] - h) / 2.0)))
#     pad.append(int(math.floor((min_dims[1] - w) / 2.0)))
#     pad.append(int(min_dims[0] - h - pad[0]))
#     pad.append(int(min_dims[1] - w - pad[1]))
#     padded_img = cv2.copyMakeBorder(img, pad[0], pad[2], pad[1], pad[3],
#                                     cv2.BORDER_CONSTANT, value=pad_value)
#     return padded_img, pad

def to_numpy(tensor):
    return tensor.detach().cpu().numpy() if tensor.requires_grad else tensor.cpu().numpy()

def preprocess(input_file, height_size, stride, pad_value=(0, 0, 0), img_mean=np.array([128, 128, 128], np.float32), img_scale=np.float32(1/256)):
    image = cv2.imread(input_file, cv2.IMREAD_COLOR)
    image = cv2.resize(image, (height_size, height_size), interpolation=cv2.INTER_LINEAR)
    # normalize_img = normalize(image, img_mean, img_scale)
    # min_dims = [height_size, max(normalize_img.shape[1], height_size)]
    # padded_img, pad = pad_width(normalize_img, stride, pad_value, min_dims)
    # tensor_img = torch.from_numpy(normalize_img).permute(2, 0, 1).unsqueeze(0).float()
    # return to_numpy(tensor_img)
    return image

input_data = [preprocess("/data1/test/301.png", height_size, stride)]
inf_results = ktc.kneron_inference(input_data, onnx_file="/data1/output2.onnx", input_names=["images"], platform=720)
print('Section 3 E2E simulator result:')
print(inf_results)

## Section 4
# Preprocess images and create the input mapping

input_images = []
for filename in os.listdir("/data1/test/"):
    if filename.endswith(".png"):
        file = "/data1/test/"+filename
        print(file)
        input_images.append(preprocess(file, height_size, stride))

input_mapping = {"images": input_images}

# Quantization
bie_path = km.analysis(input_mapping, output_bie = None, threads = 4)

# E2E simulator (fixed point)
radix = ktc.get_radix(input_images)
fixed_results = ktc.kneron_inference(input_data, bie_file=bie_path, input_names=["images"], radix=radix, platform=720)
print('Section 4 E2E simulator result:')
print(fixed_results)

## Section 5
# Batch compile
compile_result = ktc.compile([km])

# E2E simulator (hardware)
hw_results = ktc.kneron_inference(input_data, nef_file=compile_result, radix=radix, platform=720)
print('Section 5 E2E simulator result:')
print(hw_results)

